use strictures 2;

use Test::More;

use_ok('Net::Blossom');
use_ok('Net::Blossom::AuthToken');
use_ok('Net::Blossom::BlobDescriptor');
use_ok('Net::Blossom::Client');
use_ok('Net::Blossom::Error');
use_ok('Net::Blossom::PaymentRequired');
use_ok('Net::Blossom::Response');
use_ok('Net::Blossom::ServerList');
use_ok('Net::Blossom::URI');

done_testing;
